/*
Copyright 2016 Ian Jauslin

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#ifndef HH_INTEGRAL_DOUBLE_H
#define HH_INTEGRAL_DOUBLE_H

#include "double_util.h"
#include "libinum.h"

// arguments for the integral over theta
typedef struct hh_argsint1_double{
  hh_params_double params;
  array_ldouble weights;
  array_ldouble abcissa;
} hh_argsint1_double;
// arguments for the integral over rho
typedef struct hh_argsint2_double{
  hh_params_double params;
  long double theta;
} hh_argsint2_double;


// compute the integral
int hh_integrate_double(long double* out, hh_params_double params, array_ldouble abcissa, array_ldouble weights);

// integrand of the integral over theta
int hh_integrand1_double(long double* out, long double theta, void* args);

// integrand of the integral over rho
int hh_integrand2_double(long double* out, long double rho, void* args);

// derivative
int hh_d_integrate_double(long double* out, hh_params_double params, array_ldouble abcissa, array_ldouble weights);

// derivative of the integrand of the integral over theta
int hh_d_integrand1_double(long double* out, long double theta, void* args);

// derivative of the integrand of the integral over rho
int hh_d_integrand2_double(long double* out, long double rho, void* args);

#endif
